package sun.misc;

import java.util.regex.Pattern;
import java.util.Arrays;
import java.util.regex.Matcher;

public class FloatingDecimal
{
    static final int EXP_SHIFT = 52;
    static final long FRACT_HOB = 4503599627370496L;
    static final long EXP_ONE = 4607182418800017408L;
    static final int MAX_SMALL_BIN_EXP = 62;
    static final int MIN_SMALL_BIN_EXP = -21;
    static final int MAX_DECIMAL_DIGITS = 15;
    static final int MAX_DECIMAL_EXPONENT = 308;
    static final int MIN_DECIMAL_EXPONENT = -324;
    static final int BIG_DECIMAL_EXPONENT = 324;
    static final int MAX_NDIGITS = 1100;
    static final int SINGLE_EXP_SHIFT = 23;
    static final int SINGLE_FRACT_HOB = 8388608;
    static final int SINGLE_MAX_DECIMAL_DIGITS = 7;
    static final int SINGLE_MAX_DECIMAL_EXPONENT = 38;
    static final int SINGLE_MIN_DECIMAL_EXPONENT = -45;
    static final int SINGLE_MAX_NDIGITS = 200;
    static final int INT_DECIMAL_DIGITS = 9;
    private static final String INFINITY_REP = "Infinity";
    private static final int INFINITY_LENGTH;
    private static final String NAN_REP = "NaN";
    private static final int NAN_LENGTH;
    private static final FloatingDecimal.BinaryToASCIIConverter B2AC_POSITIVE_INFINITY;
    private static final FloatingDecimal.BinaryToASCIIConverter B2AC_NEGATIVE_INFINITY;
    private static final FloatingDecimal.BinaryToASCIIConverter B2AC_NOT_A_NUMBER;
    private static final FloatingDecimal.BinaryToASCIIConverter B2AC_POSITIVE_ZERO;
    private static final FloatingDecimal.BinaryToASCIIConverter B2AC_NEGATIVE_ZERO;
    private static final ThreadLocal<FloatingDecimal.BinaryToASCIIBuffer> threadLocalBinaryToASCIIBuffer;
    static final FloatingDecimal.ASCIIToBinaryConverter A2BC_POSITIVE_INFINITY;
    static final FloatingDecimal.ASCIIToBinaryConverter A2BC_NEGATIVE_INFINITY;
    static final FloatingDecimal.ASCIIToBinaryConverter A2BC_NOT_A_NUMBER;
    static final FloatingDecimal.ASCIIToBinaryConverter A2BC_POSITIVE_ZERO;
    static final FloatingDecimal.ASCIIToBinaryConverter A2BC_NEGATIVE_ZERO;
    
    public FloatingDecimal() {
    }
    
    public static String toJavaFormatString(double n) {
        return getBinaryToASCIIConverter(n).toJavaFormatString();
    }
    
    public static String toJavaFormatString(float n) {
        return getBinaryToASCIIConverter(n).toJavaFormatString();
    }
    
    public static void appendTo(double n, Appendable appendable) {
        getBinaryToASCIIConverter(n).appendTo(appendable);
    }
    
    public static void appendTo(float n, Appendable appendable) {
        getBinaryToASCIIConverter(n).appendTo(appendable);
    }
    
    public static double parseDouble(String s) throws NumberFormatException {
        return readJavaFormatString(s).doubleValue();
    }
    
    public static float parseFloat(String s) throws NumberFormatException {
        return readJavaFormatString(s).floatValue();
    }
    
    private static FloatingDecimal.BinaryToASCIIBuffer getBinaryToASCIIBuffer() {
        return (FloatingDecimal.BinaryToASCIIBuffer)FloatingDecimal.threadLocalBinaryToASCIIBuffer.get();
    }
    
    public static FloatingDecimal.BinaryToASCIIConverter getBinaryToASCIIConverter(double n) {
        return getBinaryToASCIIConverter(n, true);
    }
    
    static FloatingDecimal.BinaryToASCIIConverter getBinaryToASCIIConverter(double n, boolean b) {
        long doubleToRawLongBits = Double.doubleToRawLongBits(n);
        int n2 = (doubleToRawLongBits & Long.MIN_VALUE) != 0x0L;
        long i = doubleToRawLongBits & 0xFFFFFFFFFFFFFL;
        int n3 = (int)((doubleToRawLongBits & 0x7FF0000000000000L) >> 52);
        if (n3 == 2047) {
            if (i == 0L) {
                return n2 ? FloatingDecimal.B2AC_NEGATIVE_INFINITY : FloatingDecimal.B2AC_POSITIVE_INFINITY;
            }
            return FloatingDecimal.B2AC_NOT_A_NUMBER;
        }
        else {
            long n4;
            int n5;
            if (n3 == 0) {
                if (i == 0L) {
                    return n2 ? FloatingDecimal.B2AC_NEGATIVE_ZERO : FloatingDecimal.B2AC_POSITIVE_ZERO;
                }
                int numberOfLeadingZeros = Long.numberOfLeadingZeros(i);
                int n6 = numberOfLeadingZeros - 11;
                n4 = i << n6;
                n3 = 1 - n6;
                n5 = 64 - numberOfLeadingZeros;
            }
            else {
                n4 = (i | 0x10000000000000L);
                n5 = 53;
            }
            n3 -= 1023;
            FloatingDecimal.BinaryToASCIIBuffer binaryToASCIIBuffer = getBinaryToASCIIBuffer();
            FloatingDecimal.BinaryToASCIIBuffer.access$000(binaryToASCIIBuffer, (boolean)n2);
            FloatingDecimal.BinaryToASCIIBuffer.access$100(binaryToASCIIBuffer, n3, n4, n5, b);
            return binaryToASCIIBuffer;
        }
    }
    
    private static FloatingDecimal.BinaryToASCIIConverter getBinaryToASCIIConverter(float n) {
        int floatToRawIntBits = Float.floatToRawIntBits(n);
        int n2 = (floatToRawIntBits & Integer.MIN_VALUE) != 0x0;
        int i = floatToRawIntBits & 0x7FFFFF;
        int n3 = (floatToRawIntBits & 0x7F800000) >> 23;
        if (n3 == 255) {
            if ((long)i == 0L) {
                return n2 ? FloatingDecimal.B2AC_NEGATIVE_INFINITY : FloatingDecimal.B2AC_POSITIVE_INFINITY;
            }
            return FloatingDecimal.B2AC_NOT_A_NUMBER;
        }
        else {
            int n4;
            int n5;
            if (n3 == 0) {
                if (i == 0) {
                    return n2 ? FloatingDecimal.B2AC_NEGATIVE_ZERO : FloatingDecimal.B2AC_POSITIVE_ZERO;
                }
                int numberOfLeadingZeros = Integer.numberOfLeadingZeros(i);
                int n6 = numberOfLeadingZeros - 8;
                n4 = i << n6;
                n3 = 1 - n6;
                n5 = 32 - numberOfLeadingZeros;
            }
            else {
                n4 = (i | 0x800000);
                n5 = 24;
            }
            n3 -= 127;
            FloatingDecimal.BinaryToASCIIBuffer binaryToASCIIBuffer = getBinaryToASCIIBuffer();
            FloatingDecimal.BinaryToASCIIBuffer.access$000(binaryToASCIIBuffer, (boolean)n2);
            FloatingDecimal.BinaryToASCIIBuffer.access$100(binaryToASCIIBuffer, n3, (long)n4 << 29, n5, true);
            return binaryToASCIIBuffer;
        }
    }
    
    static FloatingDecimal.ASCIIToBinaryConverter readJavaFormatString(String trim) throws NumberFormatException {
        boolean n = false;
        boolean n2 = false;
        try {
            trim = trim.trim();
            int length = trim.length();
            if (length == 0) {
                throw new NumberFormatException("empty String");
            }
            int i = 0;
            switch (trim.charAt(i)) {
                case '-': {
                    n = true;
                }
                case '+': {
                    ++i;
                    n2 = true;
                    break;
                }
            }
            int char1 = trim.charAt(i);
            if (char1 == 'N') {
                if (length - i == FloatingDecimal.NAN_LENGTH && trim.indexOf("NaN", i) == i) {
                    return FloatingDecimal.A2BC_NOT_A_NUMBER;
                }
            }
            else if (char1 == 'I') {
                if (length - i == FloatingDecimal.INFINITY_LENGTH && trim.indexOf("Infinity", i) == i) {
                    return n ? FloatingDecimal.A2BC_NEGATIVE_INFINITY : FloatingDecimal.A2BC_POSITIVE_INFINITY;
                }
            }
            else {
                if (char1 == '0' && length > i + 1) {
                    int char2 = trim.charAt(i + 1);
                    if (char2 == 'x' || char2 == 'X') {
                        return parseHexString(trim);
                    }
                }
                char[] array = new char[length];
                int n3 = 0;
                int n4 = 0;
                int n5 = 0;
                int n6 = 0;
                int n7 = 0;
                while (i < length) {
                    int char3 = trim.charAt(i);
                    if (char3 == '0') {
                        ++n6;
                    }
                    else if (char3 == '.') {
                        if (n4 != 0) {
                            throw new NumberFormatException("multiple points");
                        }
                        n5 = i;
                        if (!(n2 == 0)) {
                            --n5;
                        }
                        n4 = 1;
                    }
                    else {
                        break;
                    }
                    ++i;
                }
                while (i < length) {
                    int char4 = trim.charAt(i);
                    if (char4 >= '1' && char4 <= '9') {
                        array[n3++] = char4;
                        n7 = 0;
                    }
                    else if (char4 == '0') {
                        array[n3++] = char4;
                        ++n7;
                    }
                    else if (char4 == '.') {
                        if (n4 != 0) {
                            throw new NumberFormatException("multiple points");
                        }
                        n5 = i;
                        if (!(n2 == 0)) {
                            --n5;
                        }
                        n4 = 1;
                    }
                    else {
                        break;
                    }
                    ++i;
                }
                int n8 = n3 - n7;
                int n9 = n8 == 0;
                if (n9 == 0 || n6 != 0) {
                    int n10;
                    if (n4 != 0) {
                        n10 = n5 - n6;
                    }
                    else {
                        n10 = n8 + n7;
                    }
                    int char5;
                    if (i < length && ((char5 = trim.charAt(i)) == 'e' || char5 == 'E')) {
                        int n11 = 1;
                        int n12 = 0;
                        int n13 = 214748364;
                        boolean n14 = false;
                        switch (trim.charAt(++i)) {
                            case '-': {
                                n11 = -1;
                            }
                            case '+': {
                                ++i;
                                break;
                            }
                        }
                        while (i < length) {
                            if (n12 >= n13) {
                                n14 = true;
                            }
                            int char6 = trim.charAt(i++);
                            if (char6 >= '0' && char6 <= '9') {
                                n12 = n12 * 10 + (char6 - 48);
                            }
                            else {
                                --i;
                                break;
                            }
                        }
                        int n15 = 324 + n8 + n7;
                        if (n14 || n12 > n15) {
                            n10 = n11 * n15;
                        }
                        else {
                            n10 += n11 * n12;
                        }
                        if (i == i) {
                            throw new NumberFormatException("For input string: \"" + trim + "\"");
                        }
                    }
                    if (i >= length || (i == length - 1 && (trim.charAt(i) == 'f' || trim.charAt(i) == 'F' || trim.charAt(i) == 'd' || trim.charAt(i) == 'D'))) {
                        if (!(n9 == 0)) {
                            return n ? FloatingDecimal.A2BC_NEGATIVE_ZERO : FloatingDecimal.A2BC_POSITIVE_ZERO;
                        }
                        return new FloatingDecimal.ASCIIToBinaryBuffer((boolean)n, n10, array, n8);
                    }
                }
            }
        }
        catch (StringIndexOutOfBoundsException ex) {}
        throw new NumberFormatException("For input string: \"" + trim + "\"");
    }
    
    static FloatingDecimal.ASCIIToBinaryConverter parseHexString(String s) {
        Matcher matcher = FloatingDecimal.HexFloatPattern.VALUE.matcher(s);
        if (!matcher.matches()) {
            throw new NumberFormatException("For input string: \"" + s + "\"");
        }
        String group = matcher.group(1);
        int n = group != null && group.equals("-");
        int length = 0;
        String group2;
        int n2;
        String s2;
        if ((group2 = matcher.group(4)) != null) {
            s2 = stripLeadingZeros(group2);
            n2 = s2.length();
        }
        else {
            String stripLeadingZeros = stripLeadingZeros(matcher.group(6));
            n2 = stripLeadingZeros.length();
            String group3 = matcher.group(7);
            length = group3.length();
            s2 = ((stripLeadingZeros == null) ? "" : stripLeadingZeros) + group3;
        }
        String stripLeadingZeros2 = stripLeadingZeros(s2);
        int length2 = stripLeadingZeros2.length();
        int n3;
        if (n2 >= 1) {
            n3 = 4 * (n2 - 1);
        }
        else {
            n3 = -4 * (length - length2 + 1);
        }
        if (length2 == 0) {
            return n ? FloatingDecimal.A2BC_NEGATIVE_ZERO : FloatingDecimal.A2BC_POSITIVE_ZERO;
        }
        String group4 = matcher.group(8);
        int n4 = group4 == null || group4.equals("+");
        long n5;
        try {
            n5 = (long)Integer.parseInt(matcher.group(9));
        }
        catch (NumberFormatException ex) {
            return n ? (n4 ? FloatingDecimal.A2BC_NEGATIVE_INFINITY : FloatingDecimal.A2BC_NEGATIVE_ZERO) : (n4 ? FloatingDecimal.A2BC_POSITIVE_INFINITY : FloatingDecimal.A2BC_POSITIVE_ZERO);
        }
        long n6 = (n4 ? 1L : -1L) * n5 + (long)n3;
        int n7 = 0;
        int n8 = 0;
        long n9 = 0L;
        long n10 = (long)getHexDigit(stripLeadingZeros2, 0);
        long n11;
        int n12;
        if (n10 == 1L) {
            n11 = (n9 | n10 << 52);
            n12 = 48;
        }
        else if (n10 <= 3L) {
            n11 = (n9 | n10 << 51);
            n12 = 47;
            ++n6;
        }
        else if (n10 <= 7L) {
            n11 = (n9 | n10 << 50);
            n12 = 46;
            n6 += 2L;
        }
        else if (n10 <= 15L) {
            n11 = (n9 | n10 << 49);
            n12 = 45;
            n6 += 3L;
        }
        else {
            throw new AssertionError("Result from digit conversion too large!");
        }
        int n13;
        for (n13 = 1; n13 < length2 && n12 >= 0; n12 -= 4, ++n13) {
            n11 |= (long)getHexDigit(stripLeadingZeros2, n13) << n12;
        }
        if (n13 < length2) {
            long n14 = (long)getHexDigit(stripLeadingZeros2, n13);
            switch (n12) {
                case -1: {
                    n11 |= (n14 & 0xEL) >> 1;
                    n7 = ((n14 & 0x1L) != 0x0L);
                    break;
                }
                case -2: {
                    n11 |= (n14 & 0xCL) >> 2;
                    n7 = ((n14 & 0x2L) != 0x0L);
                    n8 = ((n14 & 0x1L) != 0x0L);
                    break;
                }
                case -3: {
                    n11 |= (n14 & 0x8L) >> 3;
                    n7 = ((n14 & 0x4L) != 0x0L);
                    n8 = ((n14 & 0x3L) != 0x0L);
                    break;
                }
                case -4: {
                    n7 = ((n14 & 0x8L) != 0x0L);
                    n8 = ((n14 & 0x7L) != 0x0L);
                    break;
                }
                default: {
                    throw new AssertionError("Unexpected shift distance remainder.");
                }
            }
            ++n13;
            while (n13 < length2 && n8 == 0) {
                long n15 = (long)getHexDigit(stripLeadingZeros2, n13);
                n8 = (n8 != 0 || n15 != 0L);
                ++n13;
            }
        }
        int n16 = n ? Integer.MIN_VALUE : 0;
        if (n6 >= -126L) {
            if (n6 > 127L) {
                n16 |= 0x7F800000;
            }
            else {
                int n20 = 28;
                int n21 = (n11 & (1L << n20) - 1L) != 0x0L || n7 != 0 || n8 != 0;
                int n22 = (int)(n11 >>> n20);
                if ((n22 & 0x3) != 0x1 || n21) {
                    ++n22;
                }
                n16 |= ((int)n6 + 126 << 23) + (n22 >> 1);
            }
        }
        else if (n6 >= -150L) {
            int n17 = (int)(-98L - n6);
            assert n17 >= 29;
            assert n17 < 53;
            int n18 = (n11 & (1L << n17) - 1L) != 0x0L || n7 != 0 || n8 != 0;
            int n19 = (int)(n11 >>> n17);
            if ((n19 & 0x3) != 0x1 || n18) {
                ++n19;
            }
            n16 |= n19 >> 1;
        }
        float intBitsToFloat = Float.intBitsToFloat(n16);
        if (n6 > 1023L) {
            return n ? FloatingDecimal.A2BC_NEGATIVE_INFINITY : FloatingDecimal.A2BC_POSITIVE_INFINITY;
        }
        long n25;
        if (n6 <= 1023L && n6 >= -1022L) {
            n25 = ((n6 + 1023L << 52 & 0x7FF0000000000000L) | (0xFFFFFFFFFFFFFL & n11));
        }
        else {
            if (n6 < -1075L) {
                return n ? FloatingDecimal.A2BC_NEGATIVE_ZERO : FloatingDecimal.A2BC_POSITIVE_ZERO;
            }
            n8 = (n8 != 0 || n7 != 0);
            int n23 = 53 - ((int)n6 + 1074 + 1);
            assert n23 >= 1 && n23 <= 53;
            n7 = ((n11 & 1L << n23 - 1) != 0x0L);
            if (n23 > 1) {
                long n24 = ~(-1L << n23 - 1);
                n8 = (n8 != 0 || (n11 & n24) != 0x0L);
            }
            n25 = (0x0L | (0xFFFFFFFFFFFFFL & n11 >> n23));
        }
        int n26 = (n25 & 0x1L) == 0x0L;
        if ((n26 && n7 != 0 && n8 != 0) || (n26 == 0 && n7 != 0)) {
            ++n25;
        }
        return new FloatingDecimal.PreparedASCIIToBinaryBuffer(n ? Double.longBitsToDouble(n25 | Long.MIN_VALUE) : Double.longBitsToDouble(n25), intBitsToFloat);
    }
    
    static String stripLeadingZeros(String s) {
        if (!s.isEmpty() && s.charAt(0) == '0') {
            int i = 1;
            while (i < s.length()) {
                if (s.charAt(i) != '0') {
                    return s.substring(i);
                }
                ++i;
            }
            return "";
        }
        return s;
    }
    
    static int getHexDigit(String s, int n) {
        int digit = Character.digit(s.charAt(n), 16);
        if (digit <= -1 || digit >= 16) {
            throw new AssertionError("Unexpected failure of digit conversion of " + s.charAt(n));
        }
        return digit;
    }
    
    static {
        INFINITY_LENGTH = "Infinity".length();
        NAN_LENGTH = "NaN".length();
        B2AC_POSITIVE_INFINITY = new FloatingDecimal.ExceptionalBinaryToASCIIBuffer("Infinity", false);
        B2AC_NEGATIVE_INFINITY = new FloatingDecimal.ExceptionalBinaryToASCIIBuffer("-Infinity", true);
        B2AC_NOT_A_NUMBER = new FloatingDecimal.ExceptionalBinaryToASCIIBuffer("NaN", false);
        B2AC_POSITIVE_ZERO = new FloatingDecimal.BinaryToASCIIBuffer(false, new char[] { '0' });
        B2AC_NEGATIVE_ZERO = new FloatingDecimal.BinaryToASCIIBuffer(true, new char[] { '0' });
        threadLocalBinaryToASCIIBuffer = new ThreadLocal<FloatingDecimal.BinaryToASCIIBuffer>() {
            @Override
            protected FloatingDecimal.BinaryToASCIIBuffer initialValue() {
                return new FloatingDecimal.BinaryToASCIIBuffer();
            }
        };
        A2BC_POSITIVE_INFINITY = new FloatingDecimal.PreparedASCIIToBinaryBuffer(Double.POSITIVE_INFINITY, Float.POSITIVE_INFINITY);
        A2BC_NEGATIVE_INFINITY = new FloatingDecimal.PreparedASCIIToBinaryBuffer(Double.NEGATIVE_INFINITY, Float.NEGATIVE_INFINITY);
        A2BC_NOT_A_NUMBER = new FloatingDecimal.PreparedASCIIToBinaryBuffer(Double.NaN, Float.NaN);
        A2BC_POSITIVE_ZERO = new FloatingDecimal.PreparedASCIIToBinaryBuffer(0.0, 0.0f);
        A2BC_NEGATIVE_ZERO = new FloatingDecimal.PreparedASCIIToBinaryBuffer(-0.0, -0.0f);
    }
    
    private static class ExceptionalBinaryToASCIIBuffer implements FloatingDecimal.BinaryToASCIIConverter
    {
        private final String image;
        private boolean isNegative;
        
        public ExceptionalBinaryToASCIIBuffer(String image, boolean isNegative) {
            this.image = image;
            this.isNegative = isNegative;
        }
        
        @Override
        public String toJavaFormatString() {
            return this.image;
        }
        
        @Override
        public void appendTo(Appendable appendable) {
            if ((appendable instanceof StringBuilder)) {
                ((StringBuilder)appendable).append(this.image);
            }
            else if ((appendable instanceof StringBuffer)) {
                ((StringBuffer)appendable).append(this.image);
            }
            else
                assert false;
        }
        
        @Override
        public int getDecimalExponent() {
            throw new IllegalArgumentException("Exceptional value does not have an exponent");
        }
        
        @Override
        public int getDigits(char[] array) {
            throw new IllegalArgumentException("Exceptional value does not have digits");
        }
        
        @Override
        public boolean isNegative() {
            return this.isNegative;
        }
        
        @Override
        public boolean isExceptional() {
            return true;
        }
        
        @Override
        public boolean digitsRoundedUp() {
            throw new IllegalArgumentException("Exceptional value is not rounded");
        }
        
        @Override
        public boolean decimalDigitsExact() {
            throw new IllegalArgumentException("Exceptional value is not exact");
        }
    }
    
    static class BinaryToASCIIBuffer implements FloatingDecimal.BinaryToASCIIConverter
    {
        private boolean isNegative;
        private int decExponent;
        private int firstDigitIndex;
        private int nDigits;
        private final char[] digits;
        private final char[] buffer;
        private boolean exactDecimalConversion;
        private boolean decimalDigitsRoundedUp;
        private static int[] insignificantDigitsNumber;
        private static final int[] N_5_BITS;
        
        BinaryToASCIIBuffer() {
            this.buffer = new char[26];
            this.exactDecimalConversion = false;
            this.decimalDigitsRoundedUp = false;
            this.digits = new char[20];
        }
        
        BinaryToASCIIBuffer(boolean isNegative, char[] digits) {
            this.buffer = new char[26];
            this.exactDecimalConversion = false;
            this.decimalDigitsRoundedUp = false;
            this.isNegative = isNegative;
            this.decExponent = 0;
            this.digits = digits;
            this.firstDigitIndex = 0;
            this.nDigits = digits.length;
        }
        
        @Override
        public String toJavaFormatString() {
            return new String(this.buffer, 0, this.getChars(this.buffer));
        }
        
        @Override
        public void appendTo(Appendable appendable) {
            int chars = this.getChars(this.buffer);
            if ((appendable instanceof StringBuilder)) {
                ((StringBuilder)appendable).append(this.buffer, 0, chars);
            }
            else if ((appendable instanceof StringBuffer)) {
                ((StringBuffer)appendable).append(this.buffer, 0, chars);
            }
            else
                assert false;
        }
        
        @Override
        public int getDecimalExponent() {
            return this.decExponent;
        }
        
        @Override
        public int getDigits(char[] array) {
            System.arraycopy(this.digits, this.firstDigitIndex, array, 0, this.nDigits);
            return this.nDigits;
        }
        
        @Override
        public boolean isNegative() {
            return this.isNegative;
        }
        
        @Override
        public boolean isExceptional() {
            return false;
        }
        
        @Override
        public boolean digitsRoundedUp() {
            return this.decimalDigitsRoundedUp;
        }
        
        @Override
        public boolean decimalDigitsExact() {
            return this.exactDecimalConversion;
        }
        
        private void setSign(boolean isNegative) {
            this.isNegative = isNegative;
        }
        
        private void developLongDigits(int n, long detailMessage, int n2) {
            if (n2 != 0) {
                long n3 = FDBigInteger.LONG_5_POW[n2] << n2;
                long n4 = detailMessage % n3;
                detailMessage /= n3;
                n += n2;
                if (n4 >= n3 >> 1) {
                    ++detailMessage;
                }
            }
            int firstDigitIndex = this.digits.length - 1;
            if (detailMessage <= 2147483647L) {
                assert detailMessage > 0L : detailMessage;
                int n5 = (int)detailMessage;
                int j;
                int k;
                for (j = n5 % 10, k = n5 / 10; j == 0; j = k % 10, k /= 10) {
                    ++n;
                }
                while (k != 0) {
                    this.digits[firstDigitIndex--] = (char)(j + 48);
                    ++n;
                    j = k % 10;
                    k /= 10;
                }
                this.digits[firstDigitIndex] = (char)(j + 48);
            }
            else {
                int i;
                for (i = (int)(detailMessage % 10L), detailMessage /= 10L; i == 0; i = (int)(detailMessage % 10L), detailMessage /= 10L) {
                    ++n;
                }
                while (detailMessage != 0L) {
                    this.digits[firstDigitIndex--] = (char)(i + 48);
                    ++n;
                    i = (int)(detailMessage % 10L);
                    detailMessage /= 10L;
                }
                this.digits[firstDigitIndex] = (char)(i + 48);
            }
            this.decExponent = n + 1;
            this.firstDigitIndex = firstDigitIndex;
            this.nDigits = this.digits.length - firstDigitIndex;
        }
        
        private void dtoa(int n, long i, int n2, boolean b) {
            assert i > 0L;
            assert (i & 0x10000000000000L) != 0x0L;
            int numberOfTrailingZeros = Long.numberOfTrailingZeros(i);
            int n3 = 53 - numberOfTrailingZeros;
            this.decimalDigitsRoundedUp = false;
            this.exactDecimalConversion = false;
            int max = Math.max(0, n3 - n - 1);
            if (n <= 62 && n >= -21 && max < FDBigInteger.LONG_5_POW.length && n3 + FloatingDecimal.BinaryToASCIIBuffer.N_5_BITS[max] < 64 && max == 0) {
                int insignificantDigitsForPow2;
                if (n > n2) {
                    insignificantDigitsForPow2 = insignificantDigitsForPow2(n - n2 - 1);
                }
                else {
                    insignificantDigitsForPow2 = 0;
                }
                if (n >= 52) {
                    i <<= n - 52;
                }
                else {
                    i >>>= 52 - n;
                }
                this.developLongDigits(0, i, insignificantDigitsForPow2);
                return;
            }
            else {
                int estimateDecExp = estimateDecExp(i, n);
                int max2 = Math.max(0, -estimateDecExp);
                int n4 = max2 + max + n;
                int max3 = Math.max(0, estimateDecExp);
                int b2 = max3 + max;
                int n5 = n4 - n2;
                i >>>= numberOfTrailingZeros;
                int a = n4 - (n3 - 1);
                int min = Math.min(a, b2);
                int n6 = a - min;
                int n7 = b2 - min;
                int n8 = n5 - min;
                if (n3 == 1) {
                    --n8;
                }
                if (n8 < 0) {
                    n6 -= n8;
                    n7 -= n8;
                    n8 = 0;
                }
                int n9 = n3 + n6 + ((max2 < FloatingDecimal.BinaryToASCIIBuffer.N_5_BITS.length) ? FloatingDecimal.BinaryToASCIIBuffer.N_5_BITS[max2] : (max2 * 3));
                int n10 = n7 + 1 + ((max3 + 1 < FloatingDecimal.BinaryToASCIIBuffer.N_5_BITS.length) ? FloatingDecimal.BinaryToASCIIBuffer.N_5_BITS[max3 + 1] : ((max3 + 1) * 3));
                int nDigits;
                int n11;
                int n12;
                long n13;
                if (n9 < 64 && n10 < 64) {
                    if (n9 < 32 && n10 < 32) {
                        int n20 = (int)i * FDBigInteger.SMALL_5_POW[max2] << n6;
                        int n21 = FDBigInteger.SMALL_5_POW[max3] << n7;
                        int n22 = FDBigInteger.SMALL_5_POW[max2] << n8;
                        int n23 = n21 * 10;
                        nDigits = 0;
                        int detailMessage3 = n20 / n21;
                        int n24 = 10 * (n20 % n21);
                        int n25 = n22 * 10;
                        n11 = (n24 < n25);
                        n12 = (n24 + n25 > n23);
                        assert detailMessage3 < 10 : detailMessage3;
                        if (detailMessage3 == 0 && n12 == 0) {
                            --estimateDecExp;
                        }
                        else {
                            this.digits[nDigits++] = (char)(48 + detailMessage3);
                        }
                        if (!b || estimateDecExp < -3 || estimateDecExp >= 8) {
                            n11 = (n12 = 0);
                        }
                        while (n11 == 0 && n12 == 0) {
                            int detailMessage4 = n24 / n21;
                            n24 = 10 * (n24 % n21);
                            n25 *= 10;
                            assert detailMessage4 < 10 : detailMessage4;
                            if ((long)n25 > 0L) {
                                n11 = (n24 < n25);
                                n12 = (n24 + n25 > n23);
                            }
                            else {
                                n11 = 1;
                                n12 = 1;
                            }
                            this.digits[nDigits++] = (char)(48 + detailMessage4);
                        }
                        n13 = (long)((n24 << 1) - n23);
                        this.exactDecimalConversion = (n24 == 0);
                    }
                    else {
                        long n14 = i * FDBigInteger.LONG_5_POW[max2] << n6;
                        long n15 = FDBigInteger.LONG_5_POW[max3] << n7;
                        long n16 = FDBigInteger.LONG_5_POW[max2] << n8;
                        long n17 = n15 * 10L;
                        nDigits = 0;
                        int detailMessage = (int)(n14 / n15);
                        long n18 = 10L * (n14 % n15);
                        long n19 = n16 * 10L;
                        n11 = (n18 < n19);
                        n12 = (n18 + n19 > n17);
                        assert detailMessage < 10 : detailMessage;
                        if (detailMessage == 0 && n12 == 0) {
                            --estimateDecExp;
                        }
                        else {
                            this.digits[nDigits++] = (char)(48 + detailMessage);
                        }
                        if (!b || estimateDecExp < -3 || estimateDecExp >= 8) {
                            n11 = (n12 = 0);
                        }
                        while (n11 == 0 && n12 == 0) {
                            int detailMessage2 = (int)(n18 / n15);
                            n18 = 10L * (n18 % n15);
                            n19 *= 10L;
                            assert detailMessage2 < 10 : detailMessage2;
                            if (n19 > 0L) {
                                n11 = (n18 < n19);
                                n12 = (n18 + n19 > n17);
                            }
                            else {
                                n11 = 1;
                                n12 = 1;
                            }
                            this.digits[nDigits++] = (char)(48 + detailMessage2);
                        }
                        n13 = (n18 << 1) - n17;
                        this.exactDecimalConversion = (n18 == 0L);
                    }
                }
                else {
                    FDBigInteger valueOfPow52 = FDBigInteger.valueOfPow52(max3, n7);
                    int normalizationBias = valueOfPow52.getNormalizationBias();
                    FDBigInteger leftShift = valueOfPow52.leftShift(normalizationBias);
                    FDBigInteger fdBigInteger = FDBigInteger.valueOfMulPow52(i, max2, n6 + normalizationBias);
                    FDBigInteger fdBigInteger2 = FDBigInteger.valueOfPow52(max2 + 1, n8 + normalizationBias + 1);
                    FDBigInteger valueOfPow53 = FDBigInteger.valueOfPow52(max3 + 1, n7 + normalizationBias + 1);
                    nDigits = 0;
                    int quoRemIteration = fdBigInteger.quoRemIteration(leftShift);
                    n11 = ((fdBigInteger.cmp(fdBigInteger2) < 0) ? 1 : 0);
                    n12 = ((valueOfPow53.addAndCmp(fdBigInteger, fdBigInteger2) <= 0) ? 1 : 0);
                    assert quoRemIteration < 10 : quoRemIteration;
                    if (quoRemIteration == 0 && n12 == 0) {
                        --estimateDecExp;
                    }
                    else {
                        this.digits[nDigits++] = (char)(48 + quoRemIteration);
                    }
                    if (!b || estimateDecExp < -3 || estimateDecExp >= 8) {
                        n11 = (n12 = 0);
                    }
                    while (n11 == 0 && n12 == 0) {
                        int quoRemIteration2 = fdBigInteger.quoRemIteration(leftShift);
                        assert quoRemIteration2 < 10 : quoRemIteration2;
                        fdBigInteger2 = fdBigInteger2.multBy10();
                        n11 = ((fdBigInteger.cmp(fdBigInteger2) < 0) ? 1 : 0);
                        n12 = ((valueOfPow53.addAndCmp(fdBigInteger, fdBigInteger2) <= 0) ? 1 : 0);
                        this.digits[nDigits++] = (char)(48 + quoRemIteration2);
                    }
                    if (n12 != 0 && n11 != 0) {
                        fdBigInteger = fdBigInteger.leftShift(1);
                        n13 = (long)fdBigInteger.cmp(valueOfPow53);
                    }
                    else {
                        n13 = 0L;
                    }
                    this.exactDecimalConversion = (fdBigInteger.cmp(FDBigInteger.ZERO) == 0);
                }
                this.decExponent = estimateDecExp + 1;
                this.firstDigitIndex = 0;
                this.nDigits = nDigits;
                if (n12 != 0) {
                    if (n11 != 0) {
                        if (n13 == 0L) {
                            if ((this.digits[this.firstDigitIndex + this.nDigits - 1] & '\u0001') != 0x0) {
                                this.roundup();
                            }
                        }
                        else if (n13 > 0L) {
                            this.roundup();
                        }
                    }
                    else {
                        this.roundup();
                    }
                }
                return;
            }
        }
        
        private void roundup() {
            int n = this.firstDigitIndex + this.nDigits - 1;
            int n2 = this.digits[n];
            if (n2 == '9') {
                while (n2 == '9' && n > this.firstDigitIndex) {
                    this.digits[n] = '0';
                    n2 = this.digits[--n];
                }
                if (n2 == '9') {
                    ++this.decExponent;
                    this.digits[this.firstDigitIndex] = '1';
                    return;
                }
            }
            this.digits[n] = (char)(n2 + 1);
            this.decimalDigitsRoundedUp = true;
        }
        
        static int estimateDecExp(long n, int n2) {
            double n3 = (Double.longBitsToDouble(0x3FF0000000000000L | (n & 0xFFFFFFFFFFFFFL)) - 1.5) * 0.289529654 + 0.176091259 + (double)n2 * 0.301029995663981;
            long doubleToRawLongBits = Double.doubleToRawLongBits(n3);
            int n4 = (int)((doubleToRawLongBits & 0x7FF0000000000000L) >> 52) - 1023;
            int n5 = (doubleToRawLongBits & Long.MIN_VALUE) != 0x0L;
            if (n4 >= 0 && n4 < 52) {
                long n6 = 4503599627370495L >> n4;
                int n7 = (int)(((doubleToRawLongBits & 0xFFFFFFFFFFFFFL) | 0x10000000000000L) >> 52 - n4);
                return n5 ? (((n6 & doubleToRawLongBits) == 0x0L) ? (-n7) : (-n7 - 1)) : n7;
            }
            if (n4 < 0) {
                return ((doubleToRawLongBits & Long.MAX_VALUE) == 0x0L) ? 0 : (n5 ? -1 : 0);
            }
            return (int)n3;
        }
        
        private static int insignificantDigits(int n) {
            int n2;
            for (n2 = 0; (long)n >= 10L; n = (int)((long)n / 10L), ++n2) {}
            return n2;
        }
        
        private static int insignificantDigitsForPow2(int n) {
            if (n > 1 && n < FloatingDecimal.BinaryToASCIIBuffer.insignificantDigitsNumber.length) {
                return FloatingDecimal.BinaryToASCIIBuffer.insignificantDigitsNumber[n];
            }
            return 0;
        }
        
        private int getChars(char[] array) {
            assert this.nDigits <= 19 : this.nDigits;
            int n = 0;
            if (this.isNegative) {
                array[0] = '-';
                n = 1;
            }
            if (this.decExponent > 0 && this.decExponent < 8) {
                int min = Math.min(this.nDigits, this.decExponent);
                System.arraycopy(this.digits, this.firstDigitIndex, array, n, min);
                n += min;
                if (min < this.decExponent) {
                    int n5 = this.decExponent - min;
                    Arrays.fill(array, n, n + n5, '0');
                    n += n5;
                    array[n++] = '.';
                    array[n++] = '0';
                }
                else {
                    array[n++] = '.';
                    if (min < this.nDigits) {
                        int n4 = this.nDigits - min;
                        System.arraycopy(this.digits, this.firstDigitIndex + min, array, n, n4);
                        n += n4;
                    }
                    else {
                        array[n++] = '0';
                    }
                }
            }
            else if (this.decExponent <= 0 && this.decExponent > -3) {
                array[n++] = '0';
                array[n++] = '.';
                if (this.decExponent != 0) {
                    Arrays.fill(array, n, n - this.decExponent, '0');
                    n -= this.decExponent;
                }
                System.arraycopy(this.digits, this.firstDigitIndex, array, n, this.nDigits);
                n += this.nDigits;
            }
            else {
                array[n++] = this.digits[this.firstDigitIndex];
                array[n++] = '.';
                if (this.nDigits > 1) {
                    System.arraycopy(this.digits, this.firstDigitIndex + 1, array, n, this.nDigits - 1);
                    n += this.nDigits - 1;
                }
                else {
                    array[n++] = '0';
                }
                array[n++] = 'E';
                int n2;
                if (this.decExponent <= 0) {
                    array[n++] = '-';
                    n2 = -this.decExponent + 1;
                }
                else {
                    n2 = this.decExponent - 1;
                }
                if (n2 <= 9) {
                    array[n++] = (char)(n2 + 48);
                }
                else if (n2 <= 99) {
                    array[n++] = (char)(n2 / 10 + 48);
                    array[n++] = (char)(n2 % 10 + 48);
                }
                else {
                    array[n++] = (char)(n2 / 100 + 48);
                    int n3 = n2 % 100;
                    array[n++] = (char)(n3 / 10 + 48);
                    array[n++] = (char)(n3 % 10 + 48);
                }
            }
            return n;
        }
        
        static void access$000(FloatingDecimal.BinaryToASCIIBuffer binaryToASCIIBuffer, boolean sign) {
            binaryToASCIIBuffer.setSign(sign);
        }
        
        static void access$100(FloatingDecimal.BinaryToASCIIBuffer binaryToASCIIBuffer, int n, long n2, int n3, boolean b) {
            binaryToASCIIBuffer.dtoa(n, n2, n3, b);
        }
        
        static {
            FloatingDecimal.BinaryToASCIIBuffer.insignificantDigitsNumber = new int[] { 0, 0, 0, 0, 1, 1, 1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 5, 5, 5, 6, 6, 6, 6, 7, 7, 7, 8, 8, 8, 9, 9, 9, 9, 10, 10, 10, 11, 11, 11, 12, 12, 12, 12, 13, 13, 13, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16, 17, 17, 17, 18, 18, 18, 19 };
            N_5_BITS = new int[] { 0, 3, 5, 7, 10, 12, 14, 17, 19, 21, 24, 26, 28, 31, 33, 35, 38, 40, 42, 45, 47, 49, 52, 54, 56, 59, 61 };
        }
    }
    
    static class PreparedASCIIToBinaryBuffer implements FloatingDecimal.ASCIIToBinaryConverter
    {
        private final double doubleVal;
        private final float floatVal;
        
        public PreparedASCIIToBinaryBuffer(double doubleVal, float floatVal) {
            this.doubleVal = doubleVal;
            this.floatVal = floatVal;
        }
        
        @Override
        public double doubleValue() {
            return this.doubleVal;
        }
        
        @Override
        public float floatValue() {
            return this.floatVal;
        }
    }
    
    static class ASCIIToBinaryBuffer implements FloatingDecimal.ASCIIToBinaryConverter
    {
        boolean isNegative;
        int decExponent;
        char[] digits;
        int nDigits;
        private static final double[] SMALL_10_POW;
        private static final float[] SINGLE_SMALL_10_POW;
        private static final double[] BIG_10_POW;
        private static final double[] TINY_10_POW;
        private static final int MAX_SMALL_TEN;
        private static final int SINGLE_MAX_SMALL_TEN;
        
        ASCIIToBinaryBuffer(boolean isNegative, int decExponent, char[] digits, int nDigits) {
            this.isNegative = isNegative;
            this.decExponent = decExponent;
            this.digits = digits;
            this.nDigits = nDigits;
        }
        
        @Override
        public double doubleValue() {
            int min = Math.min(this.nDigits, 16);
            int n = this.digits[0] - 48;
            int i = Math.min(min, 9);
            for (int j = 1; j < i; ++j) {
                n = n * 10 + this.digits[j] - 48;
            }
            long n2 = (long)n;
            for (; i < min; ++i) {
                n2 = n2 * 10L + (long)(this.digits[i] - 48);
            }
            double n3 = (double)n2;
            int n4 = this.decExponent - min;
            if (this.nDigits <= 15) {
                if (n4 == 0 || n3 == 0.0) {
                    return this.isNegative ? (-n3) : n3;
                }
                if (n4 >= 0) {
                    if (n4 <= FloatingDecimal.ASCIIToBinaryBuffer.MAX_SMALL_TEN) {
                        double n8 = n3 * FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n4];
                        return this.isNegative ? (-n8) : n8;
                    }
                    int n6 = 15 - min;
                    if (n4 <= FloatingDecimal.ASCIIToBinaryBuffer.MAX_SMALL_TEN + n6) {
                        double n7 = n3 * FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n6] * FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n4 - n6];
                        return this.isNegative ? (-n7) : n7;
                    }
                }
                else if (n4 >= -FloatingDecimal.ASCIIToBinaryBuffer.MAX_SMALL_TEN) {
                    double n5 = n3 / FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[-n4];
                    return this.isNegative ? (-n5) : n5;
                }
            }
            if (n4 > 0) {
                if (this.decExponent > 309) {
                    return this.isNegative ? Double.NEGATIVE_INFINITY : Double.POSITIVE_INFINITY;
                }
                if ((n4 & 0xF) != 0x0) {
                    n3 *= FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n4 & 0xF];
                }
                int l;
                if ((l = n4 >> 4) != 0) {
                    int n12;
                    for (n12 = 0; l > 1; ++n12, l >>= 1) {
                        if ((l & 0x1) != 0x0) {
                            n3 *= FloatingDecimal.ASCIIToBinaryBuffer.BIG_10_POW[n12];
                        }
                    }
                    double v = n3 * FloatingDecimal.ASCIIToBinaryBuffer.BIG_10_POW[n12];
                    if (Double.isInfinite(v)) {
                        if (Double.isInfinite(n3 / 2.0 * FloatingDecimal.ASCIIToBinaryBuffer.BIG_10_POW[n12])) {
                            return this.isNegative ? Double.NEGATIVE_INFINITY : Double.POSITIVE_INFINITY;
                        }
                        v = Double.MAX_VALUE;
                    }
                    n3 = v;
                }
            }
            else if (n4 < 0) {
                int n9 = -n4;
                if (this.decExponent < -325) {
                    return this.isNegative ? -0.0 : 0.0;
                }
                if ((n9 & 0xF) != 0x0) {
                    n3 /= FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n9 & 0xF];
                }
                int k;
                if ((k = n9 >> 4) != 0) {
                    int n10;
                    for (n10 = 0; k > 1; ++n10, k >>= 1) {
                        if ((k & 0x1) != 0x0) {
                            n3 *= FloatingDecimal.ASCIIToBinaryBuffer.TINY_10_POW[n10];
                        }
                    }
                    double n11 = n3 * FloatingDecimal.ASCIIToBinaryBuffer.TINY_10_POW[n10];
                    if (n11 == 0.0) {
                        if (n3 * 2.0 * FloatingDecimal.ASCIIToBinaryBuffer.TINY_10_POW[n10] == 0.0) {
                            return this.isNegative ? -0.0 : 0.0;
                        }
                        n11 = Double.MIN_VALUE;
                    }
                    n3 = n11;
                }
            }
            if (this.nDigits > 1100) {
                this.nDigits = 1101;
                this.digits[1100] = '1';
            }
            FDBigInteger o = new FDBigInteger(n2, this.digits, min, this.nDigits);
            int b = this.decExponent - this.nDigits;
            long doubleToRawLongBits = Double.doubleToRawLongBits(n3);
            int max = Math.max(0, -b);
            int max2 = Math.max(0, b);
            FDBigInteger multByPow52 = o.multByPow52(max2, 0);
            multByPow52.makeImmutable();
            FDBigInteger leftShift = null;
            int n13 = 0;
            do {
                int n14 = (int)(doubleToRawLongBits >>> 52);
                long n15 = doubleToRawLongBits & 0xFFFFFFFFFFFFFL;
                long m;
                if (n14 > 0) {
                    m = (n15 | 0x10000000000000L);
                }
                else {
                    assert n15 != 0L : n15;
                    int n16 = Long.numberOfLeadingZeros(n15) - 11;
                    m = n15 << n16;
                    n14 = 1 - n16;
                }
                n14 -= 1023;
                int numberOfTrailingZeros = Long.numberOfTrailingZeros(m);
                long n17 = m >>> numberOfTrailingZeros;
                int n18 = n14 - 52 + numberOfTrailingZeros;
                int n19 = 53 - numberOfTrailingZeros;
                if (n18 >= 0) {
                    max += n18;
                }
                else {
                    max2 -= n18;
                }
                int n20;
                if (n14 <= -1023) {
                    n20 = n14 + numberOfTrailingZeros + 1023;
                }
                else {
                    n20 = 1 + numberOfTrailingZeros;
                }
                int a = max + n20;
                int a2 = max2 + n20;
                int min2 = Math.min(a, Math.min(a2, max));
                int n21 = a - min2;
                int n22 = a2 - min2;
                int n23 = max - min2;
                FDBigInteger valueOfMulPow52 = FDBigInteger.valueOfMulPow52(n17, max, n21);
                if (leftShift == null || n13 != n22) {
                    leftShift = multByPow52.leftShift(n22);
                    n13 = n22;
                }
                int cmp;
                int n24;
                FDBigInteger fdBigInteger;
                if ((cmp = valueOfMulPow52.cmp(leftShift)) > 0) {
                    n24 = true;
                    fdBigInteger = valueOfMulPow52.leftInplaceSub(leftShift);
                    if (n19 == 1 && n18 > -1022 && --n23 < 0) {
                        n23 = 0;
                        fdBigInteger = fdBigInteger.leftShift(1);
                    }
                }
                else if (cmp < 0) {
                    n24 = false;
                    fdBigInteger = leftShift.rightInplaceSub(valueOfMulPow52);
                }
                else {
                    break;
                }
                int cmpPow52 = fdBigInteger.cmpPow52(max, n23);
                if (cmpPow52 < 0) {
                    break;
                }
                if (cmpPow52 == 0) {
                    if ((doubleToRawLongBits & 0x1L) != 0x0L) {
                        doubleToRawLongBits += (n24 ? -1L : 1L);
                        break;
                    }
                    break;
                }
                else {
                    doubleToRawLongBits += (n24 ? -1L : 1L);
                }
            } while (doubleToRawLongBits != 0L && doubleToRawLongBits != 9218868437227405312L);
            if (this.isNegative) {
                doubleToRawLongBits |= Long.MIN_VALUE;
            }
            return Double.longBitsToDouble(doubleToRawLongBits);
        }
        
        @Override
        public float floatValue() {
            int i = Math.min(this.nDigits, 8);
            int n = this.digits[0] - 48;
            for (int j = 1; j < i; ++j) {
                n = n * 10 + this.digits[j] - 48;
            }
            float n2 = (float)n;
            int n3 = this.decExponent - i;
            if (this.nDigits <= 7) {
                if (n3 == 0 || n2 == 0.0f) {
                    return this.isNegative ? (-n2) : n2;
                }
                if (n3 >= 0) {
                    if (n3 <= FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_MAX_SMALL_TEN) {
                        float n9 = n2 * FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_SMALL_10_POW[n3];
                        return this.isNegative ? (-n9) : n9;
                    }
                    int n7 = 7 - i;
                    if (n3 <= FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_MAX_SMALL_TEN + n7) {
                        float n8 = n2 * FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_SMALL_10_POW[n7] * FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_SMALL_10_POW[n3 - n7];
                        return this.isNegative ? (-n8) : n8;
                    }
                }
                else if (n3 >= -FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_MAX_SMALL_TEN) {
                    float n6 = n2 / FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_SMALL_10_POW[-n3];
                    return this.isNegative ? (-n6) : n6;
                }
            }
            else if (this.decExponent >= this.nDigits && this.nDigits + this.decExponent <= 15) {
                long n4 = (long)n;
                for (; i < this.nDigits; ++i) {
                    n4 = n4 * 10L + (long)(this.digits[i] - 48);
                }
                float n5 = (float)((double)n4 * FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[this.decExponent - this.nDigits]);
                return this.isNegative ? (-n5) : n5;
            }
            double n10 = (double)n2;
            if (n3 > 0) {
                if (this.decExponent > 39) {
                    return this.isNegative ? Float.NEGATIVE_INFINITY : Float.POSITIVE_INFINITY;
                }
                if ((n3 & 0xF) != 0x0) {
                    n10 *= FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n3 & 0xF];
                }
                int l;
                if ((l = n3 >> 4) != 0) {
                    for (int n13 = 0; l > 0; ++n13, l >>= 1) {
                        if ((l & 0x1) != 0x0) {
                            n10 *= FloatingDecimal.ASCIIToBinaryBuffer.BIG_10_POW[n13];
                        }
                    }
                }
            }
            else if (n3 < 0) {
                int n11 = -n3;
                if (this.decExponent < -46) {
                    return this.isNegative ? -0.0f : 0.0f;
                }
                if ((n11 & 0xF) != 0x0) {
                    n10 /= FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW[n11 & 0xF];
                }
                int k;
                if ((k = n11 >> 4) != 0) {
                    for (int n12 = 0; k > 0; ++n12, k >>= 1) {
                        if ((k & 0x1) != 0x0) {
                            n10 *= FloatingDecimal.ASCIIToBinaryBuffer.TINY_10_POW[n12];
                        }
                    }
                }
            }
            float max = Math.max(Float.MIN_VALUE, Math.min(Float.MAX_VALUE, (float)n10));
            if (this.nDigits > 200) {
                this.nDigits = 201;
                this.digits[200] = '1';
            }
            FDBigInteger o = new FDBigInteger((long)n, this.digits, i, this.nDigits);
            int b = this.decExponent - this.nDigits;
            int floatToRawIntBits = Float.floatToRawIntBits(max);
            int max2 = Math.max(0, -b);
            int max3 = Math.max(0, b);
            FDBigInteger multByPow52 = o.multByPow52(max3, 0);
            multByPow52.makeImmutable();
            FDBigInteger leftShift = null;
            int n14 = 0;
            do {
                int n15 = floatToRawIntBits >>> 23;
                int n16 = floatToRawIntBits & 0x7FFFFF;
                int m;
                if (n15 > 0) {
                    m = (n16 | 0x800000);
                }
                else {
                    assert n16 != 0 : n16;
                    int n17 = Integer.numberOfLeadingZeros(n16) - 8;
                    m = n16 << n17;
                    n15 = 1 - n17;
                }
                n15 -= 127;
                int numberOfTrailingZeros = Integer.numberOfTrailingZeros(m);
                int n18 = m >>> numberOfTrailingZeros;
                int n19 = n15 - 23 + numberOfTrailingZeros;
                int n20 = 24 - numberOfTrailingZeros;
                if (n19 >= 0) {
                    max2 += n19;
                }
                else {
                    max3 -= n19;
                }
                int n21;
                if (n15 <= -127) {
                    n21 = n15 + numberOfTrailingZeros + 127;
                }
                else {
                    n21 = 1 + numberOfTrailingZeros;
                }
                int a = max2 + n21;
                int a2 = max3 + n21;
                int min = Math.min(a, Math.min(a2, max2));
                int n22 = a - min;
                int n23 = a2 - min;
                int n24 = max2 - min;
                FDBigInteger valueOfMulPow52 = FDBigInteger.valueOfMulPow52((long)n18, max2, n22);
                if (leftShift == null || n14 != n23) {
                    leftShift = multByPow52.leftShift(n23);
                    n14 = n23;
                }
                int cmp;
                int n25;
                FDBigInteger fdBigInteger;
                if ((cmp = valueOfMulPow52.cmp(leftShift)) > 0) {
                    n25 = true;
                    fdBigInteger = valueOfMulPow52.leftInplaceSub(leftShift);
                    if (n20 == 1 && n19 > -126 && --n24 < 0) {
                        n24 = 0;
                        fdBigInteger = fdBigInteger.leftShift(1);
                    }
                }
                else if (cmp < 0) {
                    n25 = false;
                    fdBigInteger = leftShift.rightInplaceSub(valueOfMulPow52);
                }
                else {
                    break;
                }
                int cmpPow52 = fdBigInteger.cmpPow52(max2, n24);
                if (cmpPow52 < 0) {
                    break;
                }
                if (cmpPow52 == 0) {
                    if ((floatToRawIntBits & 0x1) != 0x0) {
                        floatToRawIntBits += (n25 ? -1 : 1);
                        break;
                    }
                    break;
                }
                else {
                    floatToRawIntBits += (n25 ? -1 : 1);
                }
            } while (floatToRawIntBits != 0 && floatToRawIntBits != 2139095040);
            if (this.isNegative) {
                floatToRawIntBits |= Integer.MIN_VALUE;
            }
            return Float.intBitsToFloat(floatToRawIntBits);
        }
        
        static {
            SMALL_10_POW = new double[] { 1.0, 10.0, 100.0, 1000.0, 10000.0, 100000.0, 1000000.0, 1.0E7, 1.0E8, 1.0E9, 1.0E10, 1.0E11, 1.0E12, 1.0E13, 1.0E14, 1.0E15, 1.0E16, 1.0E17, 1.0E18, 1.0E19, 1.0E20, 1.0E21, 1.0E22 };
            SINGLE_SMALL_10_POW = new float[] { 1.0f, 10.0f, 100.0f, 1000.0f, 10000.0f, 100000.0f, 1000000.0f, 1.0E7f, 1.0E8f, 1.0E9f, 1.0E10f };
            BIG_10_POW = new double[] { 1.0E16, 1.0E32, 1.0E64, 1.0E128, 1.0E256 };
            TINY_10_POW = new double[] { 1.0E-16, 1.0E-32, 1.0E-64, 1.0E-128, 1.0E-256 };
            MAX_SMALL_TEN = FloatingDecimal.ASCIIToBinaryBuffer.SMALL_10_POW.length - 1;
            SINGLE_MAX_SMALL_TEN = FloatingDecimal.ASCIIToBinaryBuffer.SINGLE_SMALL_10_POW.length - 1;
        }
    }
    
    private static class HexFloatPattern
    {
        private static final Pattern VALUE;
        
        static {
            VALUE = Pattern.compile("([-+])?0[xX](((\\p{XDigit}+)\\.?)|((\\p{XDigit}*)\\.(\\p{XDigit}+)))[pP]([-+])?(\\p{Digit}+)[fFdD]?");
        }
    }
    
    interface ASCIIToBinaryConverter
    {
        public abstract double doubleValue();
        
        public abstract float floatValue();
    }
    
    public interface BinaryToASCIIConverter
    {
        public abstract String toJavaFormatString();
        
        public abstract void appendTo(Appendable p0);
        
        public abstract int getDecimalExponent();
        
        public abstract int getDigits(char[] p0);
        
        public abstract boolean isNegative();
        
        public abstract boolean isExceptional();
        
        public abstract boolean digitsRoundedUp();
        
        public abstract boolean decimalDigitsExact();
    }
}
